@@ -3,34 +3,36 @@
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+/* import-globals-from pippki.js */
 
 var keygenThread;
 
 function onLoad()
 {
   keygenThread = window.arguments[0].QueryInterface(Components.interfaces.nsIKeygenThread);
-  
+
   if (!keygenThread) {
     window.close();
     return;
   }
-  
-  setCursor("wait");
+
+  window.setCursor("wait");
 
   var obs = {
     observe : function keygenListenerObserve(subject, topic, data) {
-      if (topic == "keygen-finished")
+      if (topic == "keygen-finished") {
         window.close();
+      }
     }
   };
-  
+
   keygenThread.startKeyGeneration(obs);
 }
 
 function onClose()
 {
-  setCursor("default");
+  window.setCursor("auto");
 
-  var alreadyClosed = new Object();
+  var alreadyClosed = {};
   keygenThread.userCanceled(alreadyClosed);
 }
